#!/bin/bash

add_volume=false
install_apt=false
python_deps=false
python_reqs=(Jinja2 jmespath python-heatclient python-keystoneclient python-novaclient python-openstackclient python-swiftclient python-zunclient requests)

function setup_volume { 
	#permissions check
	if [ "$EUID" -ne 0 ]; then 
		echo "Please run with sudo"
		exit 3
	fi
	#get list of disks
	disks=$(lsblk | grep disk | cut -d ' ' -f 1)
	#select the first one with no partitions
	for disk in $disks; do 
		if (lsblk | grep -v disk | grep "$disk" >/dev/null); then 
			echo "skipping $disk"
		else 
			echo "found $disk"
			volume="$disk" 
			break
		fi
	done
	#create partition table, partition, and filesystem
	echo "using /dev/${volume}1"
	parted "/dev/${volume}" mklabel gpt
	parted "/dev/${volume}" mkpart home-volume ext4 0% 100%
	mkfs.ext4 "/dev/${volume}1"
	#move any users' home directories to the new disk
	mount "/dev/${volume}1" /mnt
	cp -rp /home/* /mnt
	umount /mnt
	#delete the original home and mount the new copy
	rm -rf /home
	mkdir /home
	mount "/dev/${volume}1" /home
	#grab volume UUID and make /etc/fstab entry
	vol_UUID=$(ls -halF /dev/disk/by-uuid/ | grep "$volume" | cut -d' ' -f11)
	echo "UUID=${vol_UUID} /home ext4 defaults 0 0" >> /etc/fstab
}

function update {
	if [ "$EUID" -ne 0 ]; then 
		echo "Please run with sudo"
		exit 4
	fi
	apt update
	apt upgrade -y
	apt install -y python3-pip qemu-system p7zip-full xauth
	pip install --upgrade pip virtualenv wheel setuptools
}

function venv_setup {
	if ! which virtualenv 2>/dev/null; then
		echo "you must have python-virtualenv installed, please rerun with -i or --install-apt"
		exit
	fi
	virtualenv "$HOME/venv"
	. "$HOME/venv/bin/activate"
	pip install --user --upgrade pip wheel setuptools
	pip install --user --upgrade "${python_reqs[@]}"
	echo . "$HOME/venv/bin/activate" >> "$HOME/.bashrc"
}

function syntax {
	echo "usage: cope [-h] [-V] [-a] [-p]"
}

function help {
	syntax
	echo ""
	echo "cope (CCTC Openstack Prepared Environment) was designed to allow easy setup of an opstation prepared to interact with Openstack."
	echo ""
	echo "-h,                   Output this help."
	echo "-V,                   Find the first disk without a partition table, and permanently"
	echo "                      mount it as /home."
	echo "-i,                   Install the apt dependencies for using the Openstack CLI in a"
	echo "                      python virtual environment."
	echo "-p,                   Create a python virtual environment and pip install packages for"
	echo "                      the Openstack CLI client."
	echo ""
}

while getopts "hVip" arg; do
	case "$arg" in
		h)           help; exit 0             ;;
		V)           add_volume=true          ;;
		i)           install_apt=true         ;;
		p)           python_deps=true         ;;
		*)           syntax; exit 1           ;;
	esac
done

if [ "$add_volume" = "true" ]; then
	setup_volume
fi
if [ "$install_apt" = "true" ]; then
	update
fi
if [ "$python_deps" =  "true" ]; then
	venv_setup
fi
